/*;  geometrc.cc {\documentstyle{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.5 $$ {\rhead{\sl geometrc.cc\quad\thepage}} */
/*;  $$Date: 1996/11/28 12:24:25 $$  */
/*;  $$Source: /home/shiiba/96prog/gtool/geometrc.cc,v $$  */

#include "geometrc.h"
#include "gtool.h"
#include "math.h"
#include <assert.h>
#include <limits.h>

// Polygon2d

Polygon2d::Polygon2d(int sizeVal)
{
  size = sizeVal;   vertex = 0;   nd = 0;
  if (size > 0) {
    vertex = new Point2d[size];
    assert(vertex);
  }
}

Polygon2d::Polygon2d(const Polygon2d& a)
{
  size = a.size;
  nd = a.nd;
  vertex = new Point2d[size];
  assert(vertex);
  int i;
  for (i = 0; i < size; i++)
    vertex[i] = a.vertex[i];
}

Polygon2d&
Polygon2d::operator=(const Polygon2d& a)
{
  if (this == &a)
    return *this;
  delete [] vertex;
  size = a.size;
  nd = a.nd;
  vertex = new Point2d[size];
  for (int i = 0; i < size; i++)
    vertex[i] = a.vertex[i];
  return *this;
}

Polygon2d::~Polygon2d()
{
  delete [] vertex;
}

void
Polygon2d::Resize(int sizeVal)
{
  delete [] vertex;
  size = sizeVal;
  vertex = new Point2d[size];
  assert(vertex);
}

Line2d
Polygon2d::Side(int i)
{
  return Line2d(vertex[i%size],vertex[(i+1)%size]); 
}

Metric
Polygon2d::Area2()
{
  Metric area = 0;
  Metric dx1, dx2, dy1, dy2;
  Point2d p0 = vertex[0];
  for (int i = 1; i <= size-2; i++) {
    dx1 = vertex[i].x() - p0.x();
    dy1 = vertex[i].y() - p0.y();
    dx2 = vertex[i+1].x() - p0.x();
    dy2 = vertex[i+1].y() - p0.y();
    area += (dx1*dy2 - dy1*dx2);
  }
  return area;
}

int
Polygon2d::Inside(Point2d a)
{
  int i;
  Point2d v1, v2;
  for (i = 0; i < size; i++) {
    v1 = vertex[i % size];   v2 = vertex[(i+1) % size];
    if (Ccw(v1, v2, a) == 0) return 0;
  }
  
  int count = 0, j = 0;
  Line2d lt, lp;
  lt.p1() = a;   lt.p2() = a;   (lt.p2()).x(INT_MAX);
  for (i = 1; i <= size; i++) {
    lp.p1() = vertex[i % size];
    lp.p2() = vertex[i % size];
    if (!Intersect(lp, lt)) {
      if (i == j+1) {
	lp.p2() = vertex[j % size];
	if (Intersect(lp,lt)) count++;
      }
      else {
	if (Ccw(lt.p1(), lt.p2(), vertex[i % size])
	    *Ccw(lt.p1(), lt.p2(), vertex[j % size]) < 0) count++;
      }
      j = i;
    }
  }
  if (count & 1) return 1;
  return -1;
}

// Polygon3d

Polygon3d::Polygon3d(int sizeVal)
{
  size = sizeVal;   vertex = 0;   nd = 0;
  if (size > 0) {
    vertex = new Point3d[size+1];
    assert(vertex);
  }
}

Polygon3d::Polygon3d(const Polygon3d& a)
{
  size = a.size; nd = a.nd;
  vertex = new Point3d[size+1];
  int i;
  for (i = 0; i < size; i++)
    vertex[i] = a.vertex[i];
  vertex[size] = vertex[0];
}

Polygon3d&
Polygon3d::operator=(const Polygon3d& a)
{
  if (this == &a)
    return *this;
  delete [] vertex;
  size = a.size; nd = a.nd;
  vertex = new Point3d[size+1];
  assert(vertex);
  for (int i = 0; i < size; i++)
    vertex[i] = a.vertex[i];
  vertex[size] = vertex[0];
  return *this;
}

Polygon3d::~Polygon3d()
{
  delete [] vertex;
}

void
Polygon3d::Resize(int sizeVal)
{
  delete [] vertex;
  size = sizeVal;
  vertex = new Point3d[size+1];
  assert(vertex);
}

Line3d
Polygon3d::Side(int i)
{
  return Line3d(vertex[i%size], vertex[(i+1)%size]);
}

Triangle2d::Triangle2d(const Triangle2d& a) {
  for (int i = 0; i < 3; i++)
    v[i] = a.v[i];
  nd = a.nd;
}

Triangle2d&
Triangle2d::operator= (const Triangle2d& a) {
  if (this == &a)
    return *this;
  for (int i = 0; i < 3; i++)
    v[i] = a.v[i];
  nd = a.nd;
  return *this;
}

Line2d
Triangle2d::Side(int i)
{
  return Line2d(v[i%3],v[(i+1)%3]);
}

Metric
Triangle2d::Area2() {
  return ::Area2(v[0], v[1], v[2]);
}

Triangle3d::Triangle3d(const Triangle3d& a) {
  for (int i = 0; i < 3; i++)
    v[i] = a.v[i];
  nd = a.nd;
}

Triangle3d&
Triangle3d::operator= (const Triangle3d& a) {
  if (this == &a)
    return *this;
  for (int i = 0; i < 3; i++)
    v[i] = a.v[i];
  nd = a.nd;
  return *this;
}

Line3d
Triangle3d::Side(int i)
{
  return Line3d(v[i%3],v[(i+1)%3]);
}

Metric
Triangle3d::HArea2() {
  return ::Area2(v[0], v[1], v[2]);
}

QuadL2d::QuadL2d(const QuadL2d& q)
{
  nd = q.nd;
  for (int i = 0; i < 4; i++)
    vd[i] = q.vd[i];
}

QuadL2d::QuadL2d(Point2d* pPtr, long nVal)
{
  nd = nVal;
  for (int i = 0; i < 4; i++)
    vd[i] = *(pPtr + i);
}

QuadL2d&
QuadL2d::operator= (const QuadL2d& q)
{
  if (this == &q)
    return *this;
  for (int i = 0; i < 4; i++)
    vd[i] = q.vd[i];
  nd = q.nd;
  return *this;
}

Line2d
QuadL2d::Side(int i)
{
  return Line2d(vd[i%4], vd[(i+1)%4]);
}

Metric
QuadL2d::Area2()
{
  return (::Area2(vd[0], vd[1], vd[2]) + ::Area2(vd[2], vd[3], vd[0]));
}

int
QuadL2d::Inside(Point2d p)
{
  int c1, c2, c3, c4;

  if ((c1 = Ccw(vd[0], vd[1], p)) == 0)
    return 0;

  if ((c2 = Ccw(vd[1], vd[2], p)) == 0)
    return 0;

  if ((c3 = Ccw(vd[2], vd[3], p)) == 0)
    return 0;

  if ((c4 = Ccw(vd[3], vd[0], p)) == 0)
    return 0;

  if (fabs(c1 + c2 + c3 + c4) == 4.)
    return 1;

  return -1;
}

QuadL3d::QuadL3d(const QuadL3d& q)
{
  nd = q.nd;
  for (int i = 0; i < 4; i++)
    vd[i] = q.vd[i];
}

QuadL3d::QuadL3d(Point3d* pPtr, long nVal)
{
  nd = nVal;
  for (int i = 0; i < 4; i++)
    vd[i] = *(pPtr + i);
}

QuadL3d&
QuadL3d::operator= (const QuadL3d& q)
{
  if (this == &q)
    return *this;
  for (int i = 0; i < 4; i++)
    vd[i] = q.vd[i];
  nd = q.nd;
  return *this;
}

Line3d
QuadL3d::Side(int i)
{
  return Line3d(vd[i%4], vd[(i+1)%4]);
}

Metric
QuadL3d::HArea2()
{
  return (::Area2(vd[0], vd[1], vd[2]) + ::Area2(vd[2], vd[3], vd[0]));
}

int
QuadL3d::Inside(Point3d p)
{
  int c1, c2, c3, c4;

  if ((c1 = Ccw(vd[0], vd[1], p)) == 0)
    return 0;

  if ((c2 = Ccw(vd[1], vd[2], p)) == 0)
    return 0;

  if ((c3 = Ccw(vd[2], vd[3], p)) == 0)
    return 0;

  if ((c4 = Ccw(vd[3], vd[0], p)) == 0)
    return 0;

  if (fabs(c1 + c2 + c3 + c4) == 4.)
    return 1;

  return -1;
}

/*; end of file */
