/*;  rseg.cc {\documentstyle{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.2 $$ {\rhead{\sl rseg.cc\quad\thepage}} */
/*;  $$Date: 1997/03/01 21:13:34 $$  */
/*;  $$Source: /home/sakakiba/96prog/slope/rseg.cc,v $$  */


#include <assert.h>
#include "rseg.h"
#include "node.h"
#include "riverpoint.h"
#include "boolean.h"

Rseg::Rseg()
{
  n = 0;
  updownFlag = 0;
  startNodeLp = NULL;
  endNodeLp = NULL;
  chaseFlag = -1; 
}

Rseg::Rseg(const Rseg& rseg)
{
  n = rseg.n;
  updownFlag = rseg.updownFlag;
  chaseFlag = rseg.chaseFlag;

  if (rseg.startNodeLp == NULL)
    startNodeLp = NULL;
  else
    {
      startNodeLp = new NodeL;
      assert(startNodeLp != NULL);
      startNodeLp = rseg.startNodeLp;
    }

  if (rseg.endNodeLp == NULL)
    endNodeLp = NULL;
  else
    {
      endNodeLp = new NodeL;
      assert(endNodeLp != NULL);
      endNodeLp = rseg.endNodeLp;
    }

  strmpList.Erase();
  strmpList = rseg.strmpList;
}

Rseg::~Rseg()
{
  if (startNodeLp != NULL)
	delete startNodeLp;
  if (endNodeLp != NULL)
	delete endNodeLp;
  if (strmpList.GetCount() > 0)
    {
      strmpList.ToHead();
      while (1)
		{
		  RiverPoint* r = strmpList.Get();
		  delete r;
		  if (strmpList.ToNext() == NO) break;
		}
    }
}

Rseg&
Rseg::operator=(const Rseg& rseg)
{
  if (&rseg == this)  return *this;

  n = rseg.n;
  updownFlag = rseg.updownFlag;
  chaseFlag = rseg.chaseFlag;

  if (startNodeLp != NULL)  delete startNodeLp;
  if (endNodeLp != NULL)  delete endNodeLp;

  if (rseg.startNodeLp == NULL)
    startNodeLp = NULL;
  else
    {
      startNodeLp = new NodeL;
      assert(startNodeLp != NULL);
      startNodeLp = rseg.startNodeLp;
    }

  if (rseg.endNodeLp == NULL)
    endNodeLp = NULL;
  else
    {
      endNodeLp = new NodeL;
      assert(endNodeLp != NULL);
      endNodeLp = rseg.endNodeLp;
    }

  strmpList.Erase();
  strmpList = rseg.strmpList;

  return *this;
}

void
Rseg::Print(ostream& dst)
{
  dst << "No." << n << " ";

  if (updownFlag == 1) dst << "from down to up";
  else  dst << "from up to down"; 

  dst << "\n";

  if (startNodeLp != NULL)
    dst << "start node = " << startNodeLp->x() << " "
	<< startNodeLp->y();
  else
    dst << "start node = NULL";
  if (endNodeLp != NULL)
    dst << "  end node = " << endNodeLp->x()
	<< " " << endNodeLp->y() << "\n";
  else
    dst << " end node = NULL\n";

  Boolean flag = YES;
  if (strmpList.GetCount() != 0) 
    {
      strmpList.ToHead();
      while (flag)
	{
	  (strmpList.Get())->Print(dst); 
	  dst << "\n";
	  flag = strmpList.ToNext();
	}
    }
  else
    {
      cerr << "Error(rseg.h) : No stream point exists" << endl;
      exit(1);
    }

  dst << "\n";
  return;
}
 
/*; end of file */
